# Generated by Django 3.1.4 on 2021-01-04 11:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('music_publisher', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cwrexport',
            name='nwr_rev',
            field=models.CharField(choices=[('NWR', 'CWR 2.1: New work registrations'), ('REV', 'CWR 2.1: Revisions of registered works'), ('NW2', 'CWR 2.2: New work registrations'), ('RE2', 'CWR 2.2: Revisions of registered works'), ('WRK', 'CWR 3.0: Work registration'), ('ISR', 'CWR 3.0: ISWC request (EDI)'), ('WR1', 'CWR 3.1 DRAFT: Work registration')], db_index=True, default='NWR', max_length=3, verbose_name='CWR version/type'),
        ),
    ]
